#include "Ethernet.h"

Ethernet::Ethernet(int lambda, int n)
{
	if(n <= 0 || lambda <= 0)
		exit(1);
		
	this->lambda = lambda;
	this->n = n;
	busy = 0;
	sentItem = -1;
		
	timeVector.resize(n);
	sentVector.resize(n);
	collidedVector.resize(n);
	contIntVector.resize(n);
	
	for(int i = 0; i < n; i++)
	{
		sentVector[i] = false;
		collidedVector[i] = false;
		contIntVector[i] = 0;
		timeVector[i] = getRandom();
	}
}

Ethernet::~Ethernet()
{}

void Ethernet::dump()
{
	COUT << ENDL;
	COUT << "Busy: " << busy << ENDL;
	COUT << "SentItem: " << sentItem << ENDL;
	
	COUT << "sent" << '\t'
		 << "col" << '\t'
		 << "Int" << '\t'
		 << "time" << '\t'
		 << ENDL;

	for(int i = 0; i < n; i++)
	{
		COUT << sentVector[i] << '\t'
			 << collidedVector[i] << '\t'
			 << contIntVector[i] << '\t'
			 << timeVector[i] << '\t'
			 << ENDL;
	}
}

float Ethernet::getRandom()
{
	return -(lambda * log( ((float)random()) / RAND_MAX ));
}

bool Ethernet::isEnded()
{
	for(int i = 0; i < n; i++)
	{
		if(!sentVector[i])
			return false;
	}
	
	return true;
}

void Ethernet::step()
{
	if(busy > 0)
		busy--;
		
	for(int i = 0; i < n; i++)
	{
		if(!sentVector[i])
		{
			timeVector[i]--;
			contIntVector[i]++;
		}
	}	
}

void Ethernet::retransmitSchedule()
{
	for(int i = 0; i < n; i++)
	{
		if(collidedVector[i])
		{
			if(sentVector[i])
			{
				contIntVector[i]++;
				sentVector[i] = false;
			}
			
			timeVector[i] = getRandom();
			collidedVector[i] = false;
		}
	}	
}

ConflictVector Ethernet::getConflicts()
{
	ConflictVector retval;
	
	for(int i = 0; i < n; i++)
	{
		if(timeVector[i] <= 0 && !sentVector[i])
			retval.push_back(i);
	}
	
	return retval;
}

void Ethernet::send()
{
	ConflictVector conflicts = getConflicts();
	
	if(conflicts.size() > 0)
	{
		if(conflicts.size() == 1 && busy == 0)
		{
			sentItem = conflicts[0];			
			sentVector[sentItem] = true;
		}
		else
		{
			for(unsigned int i = 0; i < conflicts.size(); i++)
				collidedVector[conflicts[i]] = true;
			
			if(busy > 0)
				collidedVector[sentItem] = true;

			sentItem = -1;
		}

		busy = 2;		
	}
}

void Ethernet::simulate()
{
	while(!isEnded())
	{
		dump();
		send();
		retransmitSchedule();
		step();
	}
	
	COUT << ENDL << "THE FINAL STATE IS:" << ENDL;
	dump();
}

float Ethernet::getCI()
{
	float sum = 0;
	
	for(int i = 0; i < n; i++)
		sum += contIntVector[i];
		
	return sum / n;
}
